import { map, switchMap, filter } from 'rxjs/operators';
import { Observable } from 'rxjs';

function charAt(position) {
  return function (source) {
    return source.pipe(map(function (value) {
      return value.charAt(position);
    }));
  };
}

function charCodeAt(position) {
  return function (source) {
    return source.pipe(map(function (value) {
      return value.charCodeAt(position);
    }));
  };
}

function codePointAt(position) {
  return function (source) {
    return source.pipe(map(function (value) {
      return value.codePointAt(position);
    }));
  };
}

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

function __read(o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
}

function __spread() {
    for (var ar = [], i = 0; i < arguments.length; i++)
        ar = ar.concat(__read(arguments[i]));
    return ar;
}

function concat() {
  var args = [];

  for (var _i = 0; _i < arguments.length; _i++) {
    args[_i] = arguments[_i];
  }

  var values = __spread(args);

  if (values[0] instanceof Array) {
    return function (source) {
      return source.pipe(map(function (value) {
        return value.concat.apply(value, __spread(values[0]));
      }));
    };
  } else if (values[0] instanceof Observable) {
    return function (source) {
      return values[0].pipe(switchMap(function (strings) {
        return source.pipe(map(function (value) {
          return value.concat.apply(value, __spread(strings));
        }));
      }));
    };
  } else {
    return function (source) {
      return source.pipe(map(function (value) {
        return value.concat.apply(value, __spread(values));
      }));
    };
  }
}

function endsWith(character, length) {
  return function (source) {
    return source.pipe(map(function (value) {
      return value.endsWith(character, length);
    }));
  };
}

function filterEndsWith(character, length) {
  return function (source) {
    return source.pipe(filter(function (value) {
      return value.endsWith(character, length);
    }));
  };
}

function filterIncludes(searchStr) {
  return function (source) {
    return source.pipe(filter(function (value) {
      return value.includes(searchStr);
    }));
  };
}

function filterStartsWith(character, length) {
  return function (source) {
    return source.pipe(filter(function (value) {
      return value.startsWith(character, length);
    }));
  };
}

function subscribeToCharCode(input) {
  return function (subscriber) {
    if (Array.isArray(input)) {
      subscriber.next(String.fromCharCode.apply(String, __spread(input)));
    } else {
      subscriber.next(String.fromCharCode(input));
    }

    subscriber.complete();
  };
}

function fromCharCode(input) {
  return new Observable(subscribeToCharCode(input));
}

var subscribeToCodePoint = function subscribeToCodePoint(input) {
  return function (subscriber) {
    if (Array.isArray(input)) {
      subscriber.next(String.fromCodePoint.apply(String, __spread(input)));
    } else {
      subscriber.next(String.fromCodePoint(input));
    }

    subscriber.complete();
  };
};

function fromCodePoint(input) {
  return new Observable(subscribeToCodePoint(input));
}

var subscribeToSingleOrArrayString = function subscribeToSingleOrArrayString(input) {
  return function (subscriber) {
    if (Array.isArray(input)) {
      for (var i = 0; i < input.length; i++) {
        subscriber.next(input[i]);
      }
    } else {
      subscriber.next(input);
    }

    subscriber.complete();
  };
};

function fromString(input) {
  return new Observable(subscribeToSingleOrArrayString(input));
}

var subscribeToSingleOrArrayUnicode = function subscribeToSingleOrArrayUnicode(input, form) {
  return function (subscriber) {
    if (Array.isArray(input)) {
      for (var i = 0; i < input.length; i++) {
        subscriber.next(input[i].normalize(form));
      }
    } else {
      subscriber.next(input.normalize(form));
    }

    subscriber.complete();
  };
};

function fromUnicode(input, form) {
  return new Observable(subscribeToSingleOrArrayUnicode(input, form));
}

function includes(searchStr) {
  return function (source) {
    return source.pipe(map(function (value) {
      return value.includes(searchStr);
    }));
  };
}

function indexOf(searchStr, start) {
  return function (source) {
    return source.pipe(map(function (value) {
      return value.indexOf(searchStr, start);
    }));
  };
}

function lastIndexOf(searchStr, fromIndex) {
  return function (source) {
    return source.pipe(map(function (value) {
      return value.lastIndexOf(searchStr, fromIndex);
    }));
  };
}

function mapCharCode() {
  return function (source) {
    return source.pipe(map(function (values) {
      return Array.isArray(values) ? String.fromCharCode.apply(String, __spread(values)) : String.fromCharCode(values);
    }));
  };
}

function mapCodePoint() {
  return function (source) {
    return source.pipe(map(function (values) {
      return Array.isArray(values) ? String.fromCodePoint.apply(String, __spread(values)) : String.fromCodePoint(values);
    }));
  };
}

function match(pattern) {
  return function (source) {
    return source.pipe(map(function (value) {
      return value.match(pattern);
    }));
  };
}

function matchAll(pattern) {
  return function (source) {
    return source.pipe(map(function (value) {
      return __spread(value.matchAll(pattern));
    }));
  };
}

function normalize(form) {
  return function (source) {
    return source.pipe(map(function (value) {
      return value.normalize(form);
    }));
  };
}

var PadPosition;

(function (PadPosition) {
  PadPosition["START"] = "start";
  PadPosition["END"] = "end";
})(PadPosition || (PadPosition = {}));

var TrimPosition;

(function (TrimPosition) {
  TrimPosition["START"] = "start";
  TrimPosition["END"] = "end";
  TrimPosition["ALL"] = "all";
})(TrimPosition || (TrimPosition = {}));

function padString(padPosition, maxLength, fillString) {
  return function (source) {
    return source.pipe(map(function (value) {
      switch (padPosition) {
        case PadPosition.END:
          {
            return value.padEnd(maxLength, fillString);
          }

        case PadPosition.START:
        default:
          return value.padStart(maxLength, fillString);
      }
    }));
  };
}
function padStart(maxLength, fillString) {
  return padString(PadPosition.START, maxLength, fillString);
}
function padEnd(maxLength, fillString) {
  return padString(PadPosition.START, maxLength, fillString);
}

function repeat(count, separator) {
  if (separator) {
    return function (source) {
      return source.pipe(map(function (value) {
        var output = [];

        for (var i = 0; i < count; i++) {
          output.push("" + value);
        }

        return output.join(separator);
      }));
    };
  }

  return function (source) {
    return source.pipe(map(function (value) {
      return value.repeat(count);
    }));
  };
}

function replace(pattern, replacement) {
  return function (source) {
    return source.pipe(map(function (value) {
      return value.replace(pattern, replacement);
    }));
  };
}

function reverse() {
  return function (source) {
    return source.pipe(map(function (value) {
      return __spread(value).reverse().join('');
    }));
  };
}

function search(pattern) {
  return function (source) {
    return source.pipe(map(function (value) {
      return value.search(pattern);
    }));
  };
}

function slice(startIndex, endIndex) {
  return function (source) {
    return source.pipe(map(function (value) {
      return value.slice(startIndex, endIndex);
    }));
  };
}

function split(separator, limit) {
  return function (source) {
    return source.pipe(map(function (value) {
      return value.split(separator, limit);
    }));
  };
}

function startsWith(character, length) {
  return function (source) {
    return source.pipe(map(function (value) {
      return value.startsWith(character, length);
    }));
  };
}

function substring(start, end) {
  return function (source) {
    return source.pipe(map(function (value) {
      return value.substring(start, end);
    }));
  };
}

function titleize(locales, separator) {
  if (separator === void 0) {
    separator = ' ';
  }

  return function (source) {
    return source.pipe(split(separator), map(function (values) {
      return values.map(function (word) {
        return "" + word.charAt(0).toLocaleUpperCase(locales) + word.slice(1);
      }).join(separator);
    }));
  };
}

function toLowerCase(locales) {
  return function (source) {
    return source.pipe(map(function (value) {
      return value.toLocaleLowerCase(locales);
    }));
  };
}

function toUpperCase(locales) {
  return function (source) {
    return source.pipe(map(function (value) {
      return value.toLocaleUpperCase(locales);
    }));
  };
}

function trimString(position) {
  if (position === void 0) {
    position = TrimPosition.ALL;
  }

  return function (source) {
    return source.pipe(map(function (value) {
      switch (position) {
        case TrimPosition.START:
          return value.trimLeft();

        case TrimPosition.END:
          return value.trimRight();

        case TrimPosition.ALL:
        default:
          return value.trim();
      }
    }));
  };
}
function trimLeft() {
  return trimString(TrimPosition.START);
}
function trimRight() {
  return trimString(TrimPosition.START);
}
function trim() {
  return trimString(TrimPosition.ALL);
}

var FormType;

(function (FormType) {
  FormType["NFC"] = "NFC";
  FormType["NFD"] = "NFD";
  FormType["NFKC"] = "NFKC";
  FormType["NFKD"] = "NFKD";
})(FormType || (FormType = {}));

export { FormType, PadPosition, TrimPosition, charAt, charCodeAt, codePointAt, concat, endsWith, filterEndsWith, filterIncludes, filterStartsWith, fromCharCode, fromCodePoint, fromString, fromUnicode, includes, indexOf, lastIndexOf, mapCharCode, mapCodePoint, match, matchAll, normalize, padEnd, padStart, padString, repeat, replace, reverse, search, slice, split, startsWith, substring, titleize, toLowerCase, toUpperCase, trim, trimLeft, trimRight, trimString };
